
###### Programs for dose-response modeling and benchmark dose computation ####
#               Yiliang Zhu,Jenny Jia,and Wendy Pogoda                       #
#      University of South Florida                                           # 
#                 September30, 2005                                          #
#                yzhu@hsc.usf.edu                                            #
#                (813)974-6674                                               #
##### with support from the USF Health Risk Assessment Methodology Group  ####



###############################################################################
## Part 1: All R functions required by the programs are defined here.        ##
## Upload (run) all these functions first before executing any program.      ##                                                   
###############################################################################



### Function to be used for drawing a panel-plot of dose-grouped individual outcome trajecotries 
### with log-scale of time. This is useful for revealing behavioral changes immediately after exposure. 

subj.panel<-function(x,y,subscripts, groups)
	              panel.superpose(x,y,subscripts, groups, type="b", 
	                            lty=seq(1:length(unique(groups[subscripts]))),
	                            col=seq(1:length(unique(groups[subscripts]))))
	

### Function for computing bootstrap BMD
### The following argument Arguments: 
### BtrPARA: bootstrap parameter estimate
### risk type: either additional or excess 
### FUN: the equation for BMD under a specific model
### BMDMethod: area of abnormality 
### time: the time point at which dose-response is assessment 
### spontaneous.risk: risk level of the control population 
### risk.level: BMR level or risk level increase due to exposure 
### std: an estiamte of pooled standard error
### Output: a sequence of bootstrap estimate of model parameters

BtrapBMD<-function(BtrPARA, B, risk.type = c("extra", "additional"), FUN = c(
	"ToxicoDiffusion0", "ToxicoLinear0", "ToxicoComplem0", "Diffusion0",
	"Polynomial0"), BMDMethod = c("Uppertail", "Lowertail", "Bothside"),
	time, spontaneous.risk, risk.level, threshold=NULL, controlData=NULL, std)
{
	BtrBMD <- matrix(0, length(time), B)
	for(i in 1:B) {
		if(((FUN == "ToxicoDiffusion0") || (FUN == "ToxicoLinear0") ||
			(FUN == "ToxicoComplem0") || (FUN == "Polynomial0")) &&
			(BMDMethod != "Bothside")) {
			if(BMDMethod == "Uppertail") upper.tail <- T
			else upper.tail <- F
			BMD <- BMDNormOneSide(risk.type, FUN, time, BtrPARA[
				i,  ], spontaneous.risk, risk.level, std,
				upper.tail)
		}   # for one-side risk region, use "BMDNormOneSide": closed form solution
		else { # for two-side or other models, use more general methods because of the use of iterative solution
			   # 
			BMD <- switch(BMDMethod,
				Uppertail = BMDUppertail(risk.type, FUN, time,
					BtrPARA[i,  ], spontaneous.risk, 
					risk.level, std, interval = c(0, 1), const = 0,
					threshold=threshold, controlData=controlData),
				Lowertail = BMDLowertail(risk.type, FUN, time,
					BtrPARA[i,  ], spontaneous.risk, 
					risk.level, std, interval = c(0, 1), const = 0, threshold=threshold, controlData=controlData),
				Bothside = BMDTwoSide(risk.type, FUN, time,
					BtrPARA[i,  ], spontaneous.risk, 
					risk.level, std, interval = c(0, 1), const = 0,
					threshold=threshold, controlData=controlData))
		}
		BtrBMD[, i] <- BMD
	}
	BtrBMD
}


### Calculate the boostrap BMDL based on bootstrap BMD(t) 
### BtrBMD: each row is a bootstrap BMD at various of time points 
###         for ocmputational convenience, dose are scaled to between zero and one
### Climit.level: the confidence level (e.g. 5% or 10%) used to determine empirical percentile 

BtrapBMDL<-function(BtrBMD, Climit.level)
{
	BtrBMDL <- rep(0, nrow(BtrBMD))
	BMDCounts <- rep(0, nrow(BtrBMD))
	for(i in 1:nrow(BtrBMD)) {
		temp <- BtrBMD[i,  ]
		temp <- temp[(temp < 1) & (temp > 0) & (!is.na(temp))]
		# remove all invalids
		BMDCounts[i] <- length(temp)
		BtrBMDL[i] <- quantile(temp, Climit.level)
	}
	invisible(BtrBMDL)
}

###  The following funciton computes BMD with one-sided risk. 
###  There is closed form solution to BMD under 
###  Qruadratic polynomial,Toxico-Diffusion,Linear-exponential, and Complementary-exponential model.
###  The BMD computation is based on a normal distribution around the fitted dose-response model at any given 
###  time points with a constant variance across dose and time ###  Arguments:: 
###  risk.type: additional or extra 
###  FUN: name of the fitted dose-response model
###  time: a vector of time points at which BMD is computed
###  param: parameter vector typically taken from the fitted model or their bootstrap estimates
###  spontaneous.risk: risk of the control group
###  risk.level: BMR level 
###  stdev: a constant standard deviation applicable to every dose group
###  upper.tail: logical value, if T upper tail, ortherwise lower tail, defines the risk
### threshold: alternative  
     
BMDNormOneSide<-function(risk.type, FUN, time, param, spontaneous.risk, risk.level, stdev, upper.tail=T, threshold=NULL, controlData=NULL){
	if (!is.null(threshold)){
		 if (upper.tail) spontaneous.risk<-sum(controlData>threshold)/(length(controlData)+1)
                   else spontaneous.risk<-sum(controlData<threshold)/(length(controlData)+1) 
         }
	
	if(risk.type=="extra")
	   {	if (upper.tail)  cutoff<-qnorm(c(1-spontaneous.risk, (1-spontaneous.risk)*(1-risk.level)))  
	                   else  cutoff<-qnorm(c(spontaneous.risk, 1-(1-spontaneous.risk)*(1-risk.level)))
	   }
		
	else {  if (upper.tail)  cutoff<-qnorm(c(1-spontaneous.risk, (1-spontaneous.risk-risk.level)))  
	                 else   cutoff<-qnorm(c(spontaneous.risk, (spontaneous.risk+risk.level)))
		 }
	
	tmp<-stdev*(cutoff[1]-cutoff[2])
	k<-length(param)
	if(FUN=="Polynomial0")      BMD<-tmp/((param[k-1]+param[k]*time)*time)
	if(FUN=="ToxicoDiffusion0") BMD<-tmp/((param[k-2]-param[k-1]*tmp)*time*exp(-param[k]*time))
	if(FUN=="ToxicoLinear0")    BMD<-tmp/(param[k-1]*time*exp(-param[k]*time))
    if(FUN=="ToxicoComplem0")
	   { BMD<-rep(0, length(time))
	     if((param[k-2]^2-4*param[k-2]*param[k-1]*tmp)<0) BMD<-NA
	     else {
		
	          for(i in (1:length(time))){
		           root1<-(param[k-2]-sqrt(param[k-2]^2-4*param[k-2]*param[k-1]*tmp))/(2*param[k-2]*param[k-1]*time[i]*exp(-param[k]*time[i]))
                   root2<-(param[k-2]+sqrt(param[k-2]^2-4*param[k-2]*param[k-1]*tmp))/(2*param[k-2]*param[k-1]*time[i]*exp(-param[k]*time[i]))
                   minm<-min(root1,root2)
                   maxm<-max(root1,root2)
                   if(minm>=0) BMDtmp<-minm
                    else if(maxm>=0) BMDtmp<-maxm
                      else         BMDtmp<-NA
                   BMD[i]<-BMDtmp 
                 }  # for 
             }  # else 
        } # if 
   BMD[(BMD<=0)]<-NA
   BMD
}


### The following are several functions computing BMDs under a general algorithm for any model (assuming normal distribution)
### Here a "general model" include those that are not additive between the baseline and the dose effects 
### We can test the diffusion model, for example.
###
###  BMDUppertail:  when upper tail area represents unusual outcomes
###  Arguments:: 
###  risk.type: extra or additional 
###  FUN: name of the fitted dose-response model (mean) 
###  time: a vector of time points at which BMD is computed
###  param: parameter vector typically taken from the fitted model or their bootstrap estimates
###  spontaneous.risk: risk of the control group
###  risk.level: BMR level 
###  stdev: a constant standard deviation applicable to every dose group
###  interval: the end points of an interval within which a bisection method is applied. 
### threshold: a point that separate normal from abnomral 
### controalData: outcome of control group from which an upper tail can be viewed as abnomral 
###               This is an option requiring adequate data and a good judgement of threshold
 
BMDUppertail<-function(risk.type, FUN, time, param, spontaneous.risk, risk.level, stdev, interval=c(0,1),
threshold=NULL, controlData=NULL){  
    
	if(is.character(FUN))
		FUNC<-get(FUN)
	if (!is.null(threshold)) spontaneous.risk<-sum(controlData>threshold)/(length(controlData)+1)
	   # estimate spontaneous.risk using threshold and control data
	if(risk.type=="extra") const0<-stdev*(qnorm(1-spontaneous.risk)-qnorm((1-spontaneous.risk)*(1-risk.level)))
	   else    const0<-stdev*(qnorm(1-spontaneous.risk)-qnorm(1-spontaneous.risk-risk.level))
                   
   BMD<-rep(NA, length(time))
   for(i in 1:length(time))
    {   
        const<-const0+FUNC(dose=a, time=time[i], param=param)
        temp <-uniroot(FUNC, interval=interval, time=time[i], param=param, const=const)$root
        BMD[i]<-temp
     }
   BMD
}

###  BMD when lower tail area represents unusual outcomes
###  Arguments:: identical to BMDUppertail 


BMDLowertail<-function(risk.type,FUN, time, param, spontaneous.risk, risk.level, stdev, interval=c(0,1), 
threshold=NULL, controlData=NULL){  

	if(is.character(FUN))
		FUNC <- get(FUN)
	if (!is.null(threshold)) spontaneous.risk<-sum(controlData<threshold)/(length(controlData)+1)
       # estimate spontaneous.risk from threshold and control data
 
	if(risk.type=="extra")  const0<-stdev*(qnorm(spontaneous.risk)-qnorm(1-(1-spontaneous.risk)*(1-risk.level)))
                else        const0<-stdev*(qnorm(spontaneous.risk)-qnorm(spontaneous.risk+risk.level))
                        # const0<0 
   BMD<-rep(NA, length(time))
   for(i in 1:length(time))
    {   


 	   # Use the Uniroot bisection method to find the zero(BMD)
              const<-const0+FUNC(dose=0, time=time[i], param=param)
              temp<-uniroot(FUNC, interval=interval, time=time[i], param=param, const=const)$root 
        BMD[i]<-temp
     }
   BMD
 }

###  BMD when both lower and upper tail areas represent unusual outcomes
### Currently normal distribution is assumed
###  Arguments:: 
###  risk.type: additional or extra 
###  FUN: name of the fitted dose-response model
###  time: a vector of time points at which BMD is computed
###  param: vector of estimated model parameters typically taken from the fitted model or their bootstrap estimates
###  spontaneous.risk: background risk of the control population 
###  threshold: the cut-off point below (lower tail) or above (upper tail) in which lies a proption of the control population 
### at risk (spontaneous risk)  
###  Either a spontaneous risk level or a cut-off point (quantitle) must be specified, but we can decide to let 
### quantile to superseed the spontaneous risk   
###  risk.level: BMR level 
###  stdev: a constant standard deviation imposed on every dose group (this is somewhat restricitve) 
###  interval: the end points of an interval within which a bisection method is applied. 
### controlData: if threshold is provided, spontaneous risk is determined from empirical control distribution 

BMDTwoSide<-function(risk.type, FUN = c("ToxicoDiffusion0", "ToxicoLinear0", "ToxicoComplem0", "Diffusion0","Polynomial0"), 
	time, param, spontaneous.risk=0.05, risk.level=0.05, stdev, threshold=NULL, controlData=NULL, interval=c(0,1)){
   	
   if (is.null(threshold)) cutoff<-qnorm(c(spontaneous.risk/2, 1-spontaneous.risk/2))   
		      # determining cut-off points with equal tails from the spontaneous risk if cut-off points are not given  
		else {cutoff<-qnorm(c(sum(controlData<min(threshold))/(length(controlData)+1), 
			                   1-sum(controlData>max(threshold))/(length(controlData)+1)
			                  )) 
			  spontaneous.risk<-sum(controlData<min(threshold))/(length(controlData)+1)+ 
			                    sum(controlData>max(threshold))/(length(controlData)+1)
			  }
			  # calculate spontaneous risk based on empirical distribution of the control data
			  # then converting them to normal percentile cuttoff points without calculating the mean  
			
		#  The argument "FUN" is actually the mean, i.e. the fitted dose-response model, will be used to define the BMD equation
		# "Normprob" below calculates the probability of below the lower tail or above the upper tail at given dose and time 

	if (risk.type=="extra")  const<-spontaneous.risk+(1-spontaneous.risk)*risk.level
      	else  const<-spontaneous.risk+risk.level
   BMD<-rep(NA, length(time))
   Normprob<-function(dose, FUN=FUN, time=time, param=param, stdev=stdev, cutoff=cutoff, const=const){
	          if (is.character(FUN)) FUNC <- get(FUN)
	 	      pnorm(-FUNC(dose=dose, time=time, param=param)/stdev+cutoff[1])+
	          1-pnorm(-FUNC(dose=dose, time=time, param=param)/stdev+cutoff[2])-const
           }  # Normprob
   for (i in 1:length(time)){ 
   BMD[i]<-try(uniroot(Normprob, interval=interval, FUN=FUN, time=time[i], param=param, stdev=stdev, cutoff=cutoff, const=const)$root)                   
}  # rof
   BMD  
}


### Function to plug a bootstrap sample of residuals and random effects to obtain a bootstrap estiamte of 
### of the parameters that define the mixed-effects model. 
### Arguments are defined below
### DATA: a dataframe containing four columns in the order of: 
###       ID, dose, time, residuals.
### model.list: a list containing parameter estimates as well as random effects.  
### B: the number of bootstrap samples to be taken, typically between 1000 - 2000
### dim:the numder of the model parameters
### Zcol:a vector of integer indicating which parameters have random effects
### Model: the type of model fit to the data
### BtrRandom: whether or not to bootstrap random effects
### BtrMethod: whether boostrap residual and/or random effects was done at one of the four possible levels, 
### subject vs individual data point, and across or within dose groups. 
### Spcifically, CrossData= individual data & cross dose; 
###                 CrossSubject = subject & cross dose; 
###                 CrossDataWithinDose = individual data & within dose 
###                 CrossSubjectWithinDose = subject & within dose  
###                (For the FOB data we do not recommend the 4th option.) 
### Output: bootstrap estimate of the model parameters (not including dispersion parameters) 

BtrapPARA<-function(DATA, model.list, B, dim, Zcol, 
Model=c("ToxicoDiffusion","ToxicoDiffusion1", "ToxicoDiffusion2", "ToxicoLinear","ToxicoComplem","Diffusion","Polynomial"),
                    BtrRandom=c("Yes","No"),
                    BtrMethod=c("CrossData","CrossDataWithinDose","CrossSubject","CrossSubjectWithinDose"))
       {  
          BtrPara<-matrix(0, B, dim)
          for(i in 1:B)   
            BtrPara[i,]<-Btrap.ResRan(ResData=DATA, model.list=model.list, Zcol=Zcol, Model=Model, Random=BtrRandom, Method=BtrMethod)  
          invisible(BtrPara)
        }

                     
### Bootstrap function to bootstrap estimates of the parameters
### First we need to bootstrap residuals and random effects: the case of equal cluster size is easy to implement. 
### The case with unequal cluster size is problematic if bootstrap is done at cluster level.
### A selected cluster may have fewer or more elements. One example of unequal cluster size is missing data. 
### NA is allowed but is deleted by the algorithm. 
###
###
### Argument:: 
### DATA: a dataframe containing four columns in the order of: 
###       ID, dose, time, residuals. 
### model.list: a list containing parameter estimates as well as random effects.  
### Zcol: a vector of integer indicating the position of the parameters that have associated random effects
###      For example Zcol=c(1,3) represents random effects associated with the 1st and 3rd fixed effects
### Model: one of five models allowed 
### Random: whether to bootstrap random effects 
### Method: four options corresponding to whether to bootstrap subjects or individual residuals; whether to bootstrap 
###         within or across dose-group. Bootstrap subject seems reasonable when error term are correlated with cluster; 
###         bootstrap within dose group is practical when error distribution is not the same across dose group
 
Btrap.ResRan<-function(ResData, model.list, Zcol,
            Model=c("ToxicoDiffusion","ToxicoDiffusion1", "ToxicoDiffusion2","ToxicoLinear","ToxicoComplem","Diffusion","Polynomial"), 
                       Random=c("Yes","No"), 
                       Method=c("CrossData","CrossDataWithinDose","CrossSubject","CrossSubjectWithinDose"), singular.adj=0.0001 )
{  
   if (!is.numeric(ResData$ID)) ResData$ID<-as.numeric(as.character(ResData$ID)) 
         # make sure ID is numeric 
   ResData<-as.data.frame(ResData) 
          # make sure it's a data frame so the following function is applicable 
   ResData<-ResData[!apply(ResData, 1, FUN=function(x) any(is.na(x))), ] 
          # removing rows that have any NAs 

	     # Group data according to subject
	     # subject: unique subject ID
   subject<-unique(ResData$ID)

	     # Position: Starting position of each subject in the data
   Position<-match(subject, ResData$ID)
 
         # Lsubject: length of each subject's originl data 
         # Cannot use tapply: subjects will be ordered: 
   Lsubject<-diff(Position, 1)
   Lsubject<-c(Lsubject, length(ResData$ID)-Position[length(subject)]+1)
        
         # Groupping data according to dose
  dose<-unique(ResData$dose) 

         # Starting position (row) of each dose group in the data
   Location<-match(dose, ResData$dose)

         # Length of each dose group 
   Ldose<-diff(Location, 1) 
   Ldose<-c(Ldose, length(ResData$ID)-Location[length(dose)]+1)

         
     ##############################################################################
     # The following performs boostrapping of residuals as well as random effects #
     # at 4 option levels                                                         #
     ##############################################################################
     
### With all four options the codes under "Random="Yes" were updated on Jan 17, 2006
	switch(Method, 
	
	           #####################################################################
               # Resample both residuals and random effects or resample residuals  #
              # only across dose. This approach treats individual data point      #
              # as sampling unit                                                  #
              ######################################################################

	    "CrossData"={BtrapRes<-sample(ResData$Res,replace=T)
                    if( Random=="Yes") { index<-seq(nrow(model.list$ran))   # make sure "ran" is a matrix taken from "ran$ID"
                                         Bindex<-as.vector(sample(index,replace=T))
                                         BtrapRan<-model.list$ran[Bindex, ]
                                     }  
                 # sampling of rows of random effects

                    else  BtrapRan<-model.list$ran  # copy the random effects      
                   }, 


              ############################################################
              # Resample both residuals and random effects or resample   # 
              # residuals only within each dose group                   #     
              ############################################################
  
        "CrossDataWithinDose"={
	             BtrapRes<-as.vector(unlist(tapply(ResData$Res, ResData$dose, sample, replace=T)))
                   Dindex<-ResData$dose[match(unique(ResData$ID), ResData$ID)]   # Dose level index for each subject
                   if( Random=="Yes") {  index<-seq(nrow(model.list$ran))
                                         Bindex<-as.vector(unlist(tapply(index,Dindex,sample,replace=T)))
                                         BtrapRan<-model.list$ran[Bindex,] 
                                 }
                              else      BtrapRan<-model.list$ran   # just copy the random effects 
                   }, 
               ###########################################################################################
               # Resample both subject (treat subject, not individual data point as the sampling unit    #
               # and random effects, or resample subjects only across dose group. This corresponds to    #
             # the case of serially correlated errors within subject. But the error distribution is    # 
               # identical across dose.                                                                  #                
               ###########################################################################################

        "CrossSubject"={ #Bootstrap residuals in unit of subject
               
                  BtrapRes<-BtrapCrossSubj(ResData=ResData, subject=subject, Position=Position, Lsubject=Lsubject)
                                     #Boostrap random effects cross subjects
                  if( Random=="Yes") { index<-seq(nrow(model.list$ran))
                                       Bindex<-as.vector(sample(index,replace=T))
                                       BtrapRan<-model.list$ran[Bindex,]
                                }
                  else  BtrapRan<-model.list$ran   # just copy the random effects    
               }, 

               #########################################################################
               # Resample residuals in unit of subject as well as random effects or    #
               # resample residuals in unit of subject only, within dose group         # 
               #########################################################################   
          
        "CrossSubjectWithinDose"={ # Bootstrap residuals across subjects within dose; there appears to be some problem
         
               # the essence here is to apply the program in "CrossSubject" to 
               # each dose group. This can be done by writing "CrossSubject" into 
               # a function and repeatedly call it here.  

                  BtrapRes<-rep(0, length(ResData$ID)) 
                  starting<-1                 
                  for (d in 1:length(dose)){  # do this for each dose group
                    
                     ResData.d<-ResData[ResData$dose==dose[d],]       # extracting all rows of the dose group
                     subject.d<-unique(ResData.d$ID)                 # unique subject ID of this group
                     Position.d<-match(subject.d, ResData.d$ID)       # Starting postion of each subject 
                     Lsubject.d<-diff(Position.d, 1)           
                     Lsubject.d<-c(Lsubject.d, length(ResData.d$ID)-Position.d[length(subject.d)]+1)   # the length of each subject's data               
                   
                     BtrapRes[starting:(starting+sum(Lsubject.d)-1)]<-     # New: parentheses added: (starting + sum()-1)                                                # BtrapRes[ending + 1:ending + sum(Lsubject.d)] <- BtrapCrossSubj(ResData =  ....
                     BtrapCrossSubj(ResData=ResData.d, subject=subject.d, Position=Position.d, Lsubject=Lsubject.d)
                        starting<-starting+sum(Lsubject.d)
                     } # rof

                    Dindex<-ResData$dose[match(unique(ResData$ID), ResData$ID)]   # Dose level index for each subject              
                    if( Random=="Yes") { index<-seq(nrow(model.list$ran))
                                         Bindex<-as.vector(unlist(tapply(index,Dindex,sample,replace=T)))
                                         BtrapRan<-model.list$ran[Bindex,]  
                                  }
                              else    BtrapRan<-model.list$ran 

           },  # "CrossSubjectWithinDose" 
           stop("Incorrect bootstrap speicificatiopn")
           ) # hctiws        
          BtrapRan<-as.matrix(BtrapRan)            ### added on Jan 17, 2006 
    
         ###################################################################
         #An one-step procedure to calculate the boostrap estimate of      #
         #parameters using the bootstrap residuals                         #                                                     
         ###################################################################

#### The following 4 lines to generate matrix "rand" were modified on Jan 17, 2006. Old code deleted ###### 
        
        k<-length(Zcol)     # Number of random effects
         N<-length(model.list$para) # the number of model parameters (fixed effects)
                          # This number can vary depending on the baseline and the model
         rand<-matrix(0,sum(Lsubject),N)    # Initializing the Matrix of dimension of Lsubject times N for random effects  

         for (i in 1:k)  rand[, Zcol[i]] <- rep(BtrapRan[, i], Lsubject)  # fill in only those columns corresponding to RE          
         
      # Calculate "X" matrix(derivative with respect to fixed parameters)
      # Parameter,time and dose should be scaled outside of this function

#####  Baseline can be a polynomial of order upto 2 for all models. 
#####  The case of polynomial model needs further consideration 

     t1<-model.list$time-model.list$ExpoTime   #NEW 
     t1[t1<0]<-0                               #NEW 
     Xtemp<-switch(Model,
                "ToxicoDiffusion"=ToxicoDiffusion(model.list$para[1]+rand[,1], model.list$para[2]+rand[,2], model.list$para[3]+rand[,3], model.list$para[4]+rand[,4], 
                             model.list$dose, model.list$time, t1),                            # constant baseline   
                "ToxicoDiffusion1"=ToxicoDiffusion1(model.list$para[1]+rand[,1], model.list$para[2]+rand[,2], model.list$para[3]+rand[,3], model.list$para[4]+rand[,4], 
                             model.list$para[5]+rand[,5], model.list$dose, model.list$time, t1),   # 1st order polynomial baseline
                "ToxicoDiffusion2"=ToxicoDiffusion2(model.list$para[1]+rand[,1], model.list$para[2]+rand[,2], model.list$para[3]+rand[,3], model.list$para[4]+rand[,4], 
                             model.list$para[5]+rand[,5], model.list$para[6]+rand[,6], model.list$dose, model.list$time, t1),   # 2nd order polynomial baseline
                "ToxicoLinear"=ToxicoLinear(model.list$para[1]+rand[,1], model.list$para[2]+rand[,2], model.list$para[3]+rand[,3],model.list$dose,model.list$time, t1),
                "ToxicoLinear1"=ToxicoLinear1(model.list$para[1]+rand[,1], model.list$para[2]+rand[,2], model.list$para[3]+rand[,3], 
                             model.list$para[4]+rand[,4], model.list$dose,model.list$time,t1),  # 1st order poly  baseline 
                "ToxicoLinear2"=ToxicoLinear2(model.list$para[1]+rand[,1], model.list$para[2]+rand[,2], model.list$para[3]+rand[,3], 
                             model.list$para[4]+rand[,4],  model.list$para[5]+rand[,5], model.list$dose,model.list$time, t1),  # 2nd order poly  baseline                
                "ToxicoComplem"=ToxicoComplem(model.list$para[1]+rand[,1],model.list$para[2]+rand[,2],model.list$para[3]+rand[,3], model.list$para[4]+rand[,4],model.list$dose,
                            model.list$time, t1),
                "ToxicoComplem1"=ToxicoComplem1(model.list$para[1]+rand[,1],model.list$para[2]+rand[,2],model.list$para[3]+rand[,3], 
                             model.list$para[4]+rand[,4], model.list$para[5]+rand[,5], model.list$dose,model.list$time, t1),       # 1st order polynomial 
                "ToxicoComplem2"=ToxicoComplem2(model.list$para[1]+rand[,1],model.list$para[2]+rand[,2],model.list$para[3]+rand[,3], 
                                 model.list$para[4]+rand[,4], model.list$para[5]+rand[,5], model.list$para[6]+rand[,6], 
                                 model.list$dose,model.list$time, t1),  # 2nd order polynomial
                "Diffusion"=Diffusion(model.list$para[1]+rand[,1],model.list$para[2]+rand[,2],model.list$para[3]+rand[,3],
                        model.list$para[4]+rand[,4],model.list$dose,model.list$time, t1),
                "Diffusion1"=Diffusion1(model.list$para[1]+rand[,1],model.list$para[2]+rand[,2],model.list$para[3]+rand[,3],
                         model.list$para[4]+rand[,4],model.list$para[5]+rand[,5],model.list$dose,model.list$time, t1),
                "Diffusion2"=Diffusion2(model.list$para[1]+rand[,1],model.list$para[2]+rand[,2],model.list$para[3]+rand[,3],
                         model.list$para[4]+rand[,4],model.list$para[5]+rand[,5],model.list$para[6]+rand[,6],
                         model.list$dose,model.list$time, t1),
                
                "Polynomial"=Polynomial(model.list$para[1]+rand[,1],model.list$para[2]+rand[,2], model.list$para[3]+rand[,3],model.list$para[4]+rand[,4],model.list$dose,model.list$time),
                        # no change made to poloynomial    
                 stop("Model is not correctly specified"))
         Xmatr<-as.matrix(attr(Xtemp,"gradient"))
          
         # For Diffusion Model the differentiation about D(thelast parameter) at t=0 is 0 
         if (Model=="Diffusion") Xmatr[,4][ResData$time==0]<-0
         if (Model=="Diffusion1") Xmatr[,5][ResData$time==0]<-0
         if (Model=="Diffusion2") Xmatr[,6][ResData$time==0]<-0
          

         # Calculate "Z" matrix (derivative with respect to random effects)
         # "Z" matrix depends on how many and where random parameters are there
         # and the following program only works for the case 
         # with one random parameter to the baseline intercept


         k2<-length(subject)
         Zmatr<-matrix(0, sum(Lsubject), k*k2)   # subject is a vector of unique subject ID 
                    # initialize Z matrix
                  # Position: vector of starting position of each subject's data
         for(i in 1:(k2-1))
                Zmatr[Position[i]:(Position[i+1]-1), ((i-1)*k+1):(i*k)]<-
                      Xmatr[Position[i]:(Position[i+1]-1),Zcol]
         Zmatr[Position[k2]:sum(Lsubject), ((k2-1)*k+1):(k2*k)]<-
                      Xmatr[Position[k2]:sum(Lsubject),Zcol]
         


         #Calculate "V" matrix inverse
        # Dmatr is a block diagonal matrix, each block is the variance structure for the random effects of 
        # on subject. When dose group are heterogeneous, the matrix is dividied into dose-specific block with 
        # individual scale factor and each block is a diagonal block 
        # This can be done either here or outside the call of "BtrapPara"

         Vmatr<-solve(model.list$Lambda+(Zmatr%*%model.list$Dmatr%*%t(Zmatr)))

         #Calculate T1,T2,and T3 
         T1<-t(Xmatr)%*%Vmatr%*%Xmatr
         T2<-t(Xmatr)%*%Vmatr%*%Zmatr%*%(as.vector(t(BtrapRan)))
         T3<-t(Xmatr)%*%Vmatr%*%BtrapRes
       
         # Obtainig the inverse of T1
         T1<-svd(T1)
         T1$d[T1$d < 10^(-10)]<-singular.adj   # adjust nearly singular eigenvalue to allow for inverse
         T1<-T1$v%*%diag(1/T1$d)%*%t(T1$u)
         
         BtrPara<-model.list$para+T1%*%(T3+T2)       # Obtaining boostrap parameters
         BtrPara
   } # end of Function: Btrap.ResRan

### FUN="Btrap.ResRan" calls FUN="BtrapCrossSubj" which in turn calls 
### FUN="BtrapResAssign"
### Function: Generate one sample bootstrap residuals ###
### Argument:: 
### subject: vector of unique ID of subjects
### ResData: a stacked vector of residuals composed of subvectors of residuals of individual subjects 
### Position: Starting row of each subject's data 
### Lsubject: lenghth of each subject data 
 
BtrapCrossSubj<-function(ResData, subject, Position, Lsubject)
{
	             Btrapsubject<-sample(subject, replace=T)   
                    # bootstrap sample of subjects' ID

                 BtrapPosition<-match(Btrapsubject,subject)
                    # The gives the position of the subject in the ID vector. This position determines where to
                    # get the sampled subjects. 
                                  
               BtrapResIndex<-as.vector(unlist(apply(cbind(Position[BtrapPosition], Lsubject[BtrapPosition]),1,
                          FUN=function(x) seq(x[1], length=x[2]))))   # this generates the index of data from the sampled subjects

               ### extracting the (bootstrap) residuals and time from the dataframe "ResData" 
               BootstrapList<-split(ResData[BtrapResIndex,3:4],   
                     rep(1:length(BtrapPosition), Lsubject[BtrapPosition]))

               # Similarly, we need an index vector for the original residuals 
               # in a list. But this needs to be executed only once. 
               
               ResIndex<-as.vector(unlist(apply(cbind(Position, Lsubject), 1, 
                               FUN=function(x) seq(x[1], length=x[2]))))
               OrigList<-split(ResData[ResIndex, 3:4],   
                               rep(1:length(Position), Lsubject))
               # Assign bootstrap residuals to replace the original ones with 
               # subject as unit

# ResData$Res is the original residuals and should be kept unchanged
# BootstrapList$BtrapRes is the sampled residuals 
# OrigList$BtrapRes is the bootstrap residuals after adjusting for difference in length 
# of residuals between the original and bootstrapped subjects, and is 
# used for estimating bootstrap estimate of regression parameters  

               for (i in 1:length(OrigList)) OrigList[[i]]$BtrapRes<-BtrapResAssign(OrigList[[i]]$BtrapRes, BootstrapList[[i]]$BtrapRes,
                                  OrigList[[i]]$time, BootstrapList[[i]]$time)
               BtrapRes<-as.vector(unlist(lapply(OrigList, FUN=function(x) x$BtrapRes)))
               invisible(BtrapRes)
  
} # end of function BtrapCrossSubj   
        
        
### FUNCTION="BtrapResAssign" 
### assigns the bootstrap residuals at subject-level to each subject. 
### The main issue is that the bootstrap residual and the original residual 
### may be of different length. The strategy adopted here is to identify 
### the intersection of the original and bootstrap residuals, only replace   
### the original one when the boostrap residual corresponds to the same time 
### point. For example, if the origianl residuals are avaiable at time (0,24)
### for the particular subject (i.e. missing data), 
### and the bootstrap residuals are available at time (0,2, 24, 168), 
### only the first (time=0) and the third (time=24) are used to replace the 
### orginal ones. 
  
### Argument: Res= vector of original residuals from a subject 
###           BtrapRes= vector of bootstrap residuals from another subject
###           TimeRes = the time vector for the subject 
###           TimeBRes = time vector for the bootstrap subject

### The argument must be obtained prior to the call of this function. 
### the best way to apply this function is through "lapply"               
  
BtrapResAssign<-function(Res, BRes, TimeRes, TimeBRes){
              Res[!is.na(match(TimeRes, TimeBRes))]<-BRes[!is.na(match(TimeBRes, TimeRes))]
              Res
}



### ToxicoDiffusion model used in model fitting ###

toxico.diffusion<-function(A, B, C, K, time, ExpoTime){  # for model fitting 
	TruncTime <- time - ExpoTime
	TruncTime[TruncTime < 0] <- 0
	A + (B * TruncTime * exp( - K * TruncTime))/(1 + C * TruncTime * exp(
		 - K * TruncTime))
}

### Defintions of Models and their derivatives with respect to fixed parameters and random effects
### These are used in bootstrap 

ToxicoDiffusion<-function(A,B,C,K,d,t,ExpoTime){
		t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
       A+B*d*t1*exp(-K*t1)/(1+C*d*t1*exp(-K*t1))
       }

ToxicoDiffusion<-deriv(~A+B*d*t1*exp(-K*t1)/(1+C*d*t1*exp(-K*t1)),c("A","B","C","K"),
                        function(A,B,C,K,d,t,t1){},formal=T)

ToxicoDiffusion1<-function(A0,A1,B,C,K,d,t, ExpoTime){
	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
   A0+A1*t+B*d*t1*exp(-K*t1)/(1+C*d*t1*exp(-K*t1))
   }
ToxicoDiffusion1<-deriv(~A0+A1*t+B*d*t1*exp(-K*t1)/(1+C*d*t1*exp(-K*t1)),c("A0","A1", "B","C","K"),
                        function(A0,A1,B,C,K,d,t,t1){},formal=T)

ToxicoDiffusion2<-function(A0,A1,A2,B,C,K,d,t, ExpoTime){
	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
   A0+A1*t+A2*t^2+B*d*t1*exp(-K*t1)/(1+C*d*t1*exp(-K*t1))}
ToxicoDiffusion2<-deriv(~A0+A1*t+A2*t^2+B*d*t1*exp(-K*t1)/(1+C*d*t1*exp(-K*t1)),c("A0","A1", "A2", "B","C","K"),
                        function(A0,A1,A2,B,C,K,d,t, t1){},formal=T)

### ToxicoLinear: will be used in future version ###

ToxicoLinear<-function(A,B,K,d,t,ExpoTime){
	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
	A+B*d*t1*exp(-K*t1)
   }
ToxicoLinear<-deriv(~A+B*d*t1*exp(-K*t1),c("A","B","K"),
                     function(A,B,K,d,t,t1){},formal=T)

ToxicoLinear1<-function(A,A1,B,K,d,t, ExpoTime){
	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
A0+A1*t+B*d*t1*exp(-K*t1)
}
ToxicoLinear1<-deriv(~A0+A1*t+B*d*t1*exp(-K*t1),c("A0","A1","B","K"),
                     function(A0,A1,B,K,d,t,t1){},formal=T)

ToxicoLinear2<-function(A0,A1,A2,B,K,d,t, ExpoTime){
	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
A0+A1*t+A2*t^2+B*d*t1*exp(-K*t1)
}
ToxicoLinear2<-deriv(~A0+A1*t+A2*t^2+B*d*t1*exp(-K*t1),c("A0","A1","A2", "B","K"),
                     function(A0,A1,A2,B,K,d,t,t1){},formal=T)


### ToxicoComplem: will be used in future version ###
 ToxicoComplem<-function(A,B,C,K,d,t,ExpoTime){
	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
	A+B*d*t1*exp(-K*t1)*(1-C*d*t1*exp(-K*t1))
   }
ToxicoComplem<-deriv(~A+B*d*t1*exp(-K*t1)*(1-C*d*t1*exp(-K*t1)),c("A","B","C","K"),
                      function(A,B,C,K,d,t,t1){},formal=T)

ToxicoComplem1<-function(A0,A1,B,C,K,d,t, ExpoTime){
	t1<-iselse(t<=ExpoTime, 0, t-ExpoTime)
   A0+A1*t+B*d*t1*exp(-K*t1)*(1-C*d*t1*exp(-K*t1))
}
ToxicoComplem1<-deriv(~A0+A1*t+B*d*t1*exp(-K*t1)*(1-C*d*t1*exp(-K*t1)),c("A0","A1","B","C","K"),
                      function(A0,A1,B,C,K,d,t,t1){},formal=T)

ToxicoComplem2<-function(A0,A1,A2,B,C,K,d,t, ExpoTime){
	t1<-iselse(t<=ExpoTime, 0, t-ExpoTime)
   A0+A1*t+A2*t^2+B*d*t1*exp(-K*t1)*(1-C*d*t1*exp(-K*t1))
}
ToxicoComplem2<-deriv(~A0+A1*t+A2*t^2+B*d*t1*exp(-K*t1)*(1-C*d*t1*exp(-K*t1)),c("A0","A1","A2","B","C","K"),
                      function(A0,A1,A2,B,C,K,d,t,t1){},formal=T)

### Diffusion: will be used in future version ###
### The diffusion function needs modification to account for ExpoTime!=0: Not done yet ###### 

Diffusion<-function(A,B,C,D,d,t,ExpoTime){
   	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
	(A+B*d*t1)/(1+exp(C)*d*t1^D)
}
Diffusion<-deriv(~(A+B*d*t1)/(1+exp(C)*d*t1^D),c("A","B","C","D"),
                  function(A,B,C,D,d,t,t1){},formal=T)

Diffusion1<-function(A,B1,B2,C,D,d,t, ExpoTime){
	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)
   (A+B1*t+B2*d*t1)/(1+exp(C)*d*t1^D)
}
Diffusion1<-deriv(~(A+B1*t+B2*d*t1)/(1+exp(C)*d*t1^D),c("A","B1","B2","C","D"),
                  function(A,B1,B2,C,D,d,t,t1){},formal=T)

Diffusion2<-function(A,B1,B2,B3,C,D,d,t, ExpoTime){
	t1<-ifelse(t<=ExpoTime, 0, t-ExpoTime)   (A+B1*t+B2*t^2+B3*d*t1)/(1+exp(C)*d*t1^D)
}
Diffusion2<-deriv(~(A+B1*t+B2*t^2+B3*d*t1)/(1+exp(C)*d*t1^D),c("A","B1","B2","B3","C","D"),
                  function(A,B1,B2,B3,C,D,d,t,t1){},formal=T)


### Polynomial ###

Polynomial<-function(A,B,C,dose,time, ExpoTime){
	t1<-ifelse(time<=ExpoTime, 0, time-ExpoTime)
   A+B*dose*t1+C*dose*t1^2
}
Polynomial<-deriv(~A+B*dose*t1+C*dose*t1^2,c("A","B","C"),
                   function(A,B,C,dose, time, ExpoTime){},formal=T)

Polynomial1<-function(A,B1,B2,C,dose,time, ExpoTime){
	t1<-ifelse(time<=ExpoTime, 0, time-ExpoTime)
   A+B1*time+B2*dose*t1+C*dose*t1^2
}
Polynomial1<-deriv(~A+B1*time+B2*dose*t1+C*dose*t1^2,c("A","B1","B2","C"),
                   function(A,B1,B2,C,dose, time, ExpoTime){},formal=T)

Polynomial2<-function(A,B1,B2,C1,C2, dose,time, ExpoTime){
	t1<-ifelse(time<=ExpoTime, 0, time-ExpoTime)
   A+B1*time+C1*time^2+B2*dose*t1+C2*dose*t1^2
}
Polynomial2<-deriv(~A+B1*time+C1*time^2+B2*dose*t1+C2*dose*t1^2,c("A","B1","B2","C1","C2"),
                   function(A,B1,B2,C1,C2, dose, time, ExpoTime){},formal=T)


### Definition of Models for the purpose of calculating Benchmark dose 
### since baseline will be cancelled out in the BMD calculaiton, we don not need to define the 
### baseline part in ToxicDiffusion0. This can be done in each of these functions except for 
### diffusion function which is not additive between the baseline and the dose component
### The "const" term is used in these funciton to facilitate the conversion of the BMD equation 
### to a difference of the mean (dose-response model) between BMD and dose=0. 
### For example, for the toxico-diffusion, linear exponential and complementary exponential model, 
### the BMD equation reduces to "mu(BMD, time, param)-mu(0, time, param)=const". 
### Because of the additive nature of these models, "mu(BMD, time, parma)=mu(0, time, param)+f(BMD, time, param)" 
### So the equation to solve for BMD becomes f(BMD, time, param)=const. 
### If the model is not additive (e.g. diffusion model) then the entire difference should be used. 

### Functions defined here are only for solving for BMD via "uniroot"
### The baseline, which is additive to the dose-effects, is removed from the definition
### This way, the order of polynomial for baseline needs not be considered. 
 
ToxicoDiffusion0<-function(dose, time, param, const=0){
	param[length(param)-2]*time*dose*exp(-param[length(param)]*time)/(1+param[length(param)-1]*dose*time*exp(-param[length(param)]*time))-const
}


ToxicoLinear0<-function(dose, time, param, const=0){
	param[2]*time*dose*exp(-param[3]*time)-const
}

ToxicoComplem0<-function(dose, time, param, const=0){
	param[2]*time*dose*exp(-param[4]*time)*(1-param[3]*time*dose*exp(-param[4]*time))-const
}

Diffusion0<-function(dose, time, param, const=0){
	(param[1]+(param[2]+param[3]*dose)*time)/(1+exp(param[4]*dose)*time^param[5])-const
}

Polynomial0<-function(dose, time, param, const=0){
	param[1]+(param[2]+param[3]*dose)*time+(param[4]*dose)*time^2-const
}

#####################################################################################################

#################################################################################
# This algorithm determines the initial Values for the Toxico Diffusion model:  #
# y=A(t)+B*d*t*exp(-K*t)/(1+C*d*t*exp(-K*t))                                    #         
# While it is possible the solve for (A,B,C,K) mathematically                   #
# with data at exactly 4 design points which gives 4 equations.                 # 
# However, with D doses and T time points, there are more                       #
# equations than parameters. There is a suggestion in the                       # 
# literature that a non-linear regression model is fit to the                   # 
# data first, and then use the resulting estimate as the initial                #
# value for the case with random effects. However, initial value                #
# is needed to fit the nonlinear regression model.                              #
#                                                                               #
#    Yiliang Zhu, University of South Florida                                   #
#################################################################################

# Assuming data are given in a data.frame "data" with ID=data[,1], dose=data[,2], 
# time=data[,3], and Y=data[,4] in a data.frame of 4 columns with the given order.

# Here we require that exposure time be given, and study begin at 0 time.
# Two versions of time scale are needed: the baseline model A(t) utilizes the original 
# (actual) time; the alteration component of the model must utilize a truncated time, 
# t*=0 (when t< ExpoTime) and t*=t-expTime (when t> expTime). 

# The baseline A(t) is typically a constant or a lower order polynomial
# Thus the intercept A is easy to estimate using data of d(dose)=0 or t(time)<=exposure time 
# The remaining three parameters need to be estimated based on data for which 
# d>0 and t> expousre time.

# 1. Getting and processing the data: assume "data" contains all necessary 
#    component; pay attention to the order of the variables or make necessary change 
#    depending on the data input format 
 

ToxicoDiffInit<-function(data, ExpoTime=0, baselinePoly=1){
       ID<-data[,1]
       Dose<-data[,2]
       Time<-data[,3] 
       y<-data[,4]

	TTime<-Time-ExpoTime
    if (ExpoTime !=0) TTime[TTime <= 0]<-0

# Obtaining theinitial value of polynomial coefficients 
# That is, baseline(t)=A0+A1*t+A2*t^2 
# computed at three time points

if (baselinePoly==1) A.hat<-mean(y[TTime==0 | Dose==0], na.rm=T) 
if (baselinePoly==2) 
	  A.hat<-lm(y[TTime==0|Dose==0]~Time[TTime==0|Dose==0], na.action=na.exclude)$coef
if (baselinePoly==3)
	  A.hat<-lm(y[TTime==0|Dose==0]~Time[TTime==0|Dose==0]+I(Time[TTime==0|Dose==0])^2, na.action=na.exclude)$coef
	
################## Need to consider other models 

# For each set of three equations (e.g. three non-baseline time points at a 
# given dose level d: t1,t2,t3>0)  
# (1) calculate the three average value (y1,y2,y3) 
# corresponding to the sub-sample (d,t1*), (d,t2*), and (d,t3*); 
# (2) Set up three equations 
#     y1-baseline(t1)=B*d*t1*exp(-K*t1)/(1+C*d*t1*exp(-K*t1))
#     y2-baseline(t2)=B*d*t2*exp(-K*t2)/(1+C*d*t2*exp(-K*t2))
#     y3-baseline(t3)=B*d*t3*exp(-K*t3)/(1+C*d*t3*exp(-K*t3))
# (3) Solve the equations to get (B,C,K) 
#     - given K, obtain (B,C) from the linear system 
#     - given (B,C), solve for K
#     - interate between (B,C) and K until convergence
 
# Consider other ways of selecting three equations? 
# For example selecting (same) time points from different dose level? 
# This alternative may work for (B, C). For estimating K different time points 
# is necessary. 
# A specific consideration is to selected the TOPEs at the two highest dose levels 
# and to select the next time point at the highest dose level.
  
# For the FOB experiments, repeat the above process starting from the highest 
# to the lowest dose that demonstrates dose effects, circulating the time points, 
# from lower to higher. Example, suppose there are
# only two time points (t1, t2) at each dose level, we take 
#       (Maximum dose, t2), (Maximum dose, t1), (next highest dose, t2) 
# to solve for (B,C,K); then take 
#        (Next highest dose, t1), (following highest dose, t2), (folowing highest dose, t1)

#### QUESTION  ####
# In the case there are less than three distinct time points at any 
# given dose level, what would happen if we combine the equations from different 
# dose groups? As an example, suppose there are only two time points but we can utilize 
# at least two dose groups, so there are at least four equations

#     y1(d1)-baseline(t1)=B*d1*t1*exp(-K*t1)/(1+C*d1*t1*exp(-K*t1))
#     y2(d1)-baseline(t2)=B*d1*t2*exp(-K*t2)/(1+C*d1*t2*exp(-K*t2))
#     y1(d2)-baseline(t1)=B*d2*t1*exp(-K*t1)/(1+C*d2*t1*exp(-K*t1))
#     y2(d2)-baseline(t2)=B*d2*t2*exp(-K*t2)/(1+C*d2*t2*exp(-K*t2))

# So long as d1*t2 != d2*t1 we will have 4 distinct equations? 

N.Dose<-length(unique(Dose[Dose>0]))  # number of dose groups excluding control 
N.Time<-length(unique(Time[TTime>0])) # number of the distinct timepoints excluding those leading up to the baseline

TD<-N.Dose*N.Time                # Number of distinct design points (dose>0 and TTime>0)
                                      # these points will be selected to set up the equations
D.group<-as.vector(tapply(Dose[TTime>0 & Dose>0], list(Time[TTime>0 & Dose>0],Dose[TTime>0 & Dose>0]), unique))
                                      # Choose those unique dose points at their first occurrence

T.group<-as.vector(tapply(TTime[TTime>0 & Dose>0], list(Time[TTime>0 & Dose>0],Dose[TTime>0 & Dose>0]), unique))
                                      # Choose those unique time points at their first occurrence  

y.mean<-as.vector(tapply(y[TTime>0 & Dose>0], list(Time[TTime>0 & Dose>0],Dose[TTime>0 & Dose>0]), mean, na.rm=T))
                                      # Average value of outcome for all non-baseline dose-time combinations 
                                     # The corresponding dose*time may not be unique, but does not appear to be an issue

InitValMatrix<-matrix(rep(NA, (3+baselinePoly)*round(TD/3)), round(TD/3), 3+baselinePoly) 
      # the output matrix of initial values organized by row; 
      # allow for a polynomial control trajectory with multiple coefficients

for (i in (1:(floor(TD/3)))){
    Time.tmp<-T.group[(TD-i*3+1):(TD-(i-1)*3)]    # choose three time points  
    baseline<-switch(baselinePoly,
                     A.hat,
                     A.hat[1]+A.hat[2]*Time.tmp,
                     A.hat[1]+A.hat[2]*Time.tmp+A.hat[3]*(Time.tmp)^2,
                     stop("Model for control trajecotry incorrect or not supported")) 
    y.tmp<-y.mean[(TD-i*3+1):(TD-(i-1)*3)]-baseline  # average response change at the three points
    Dose.tmp<-D.group[(TD-i*3+1):(TD-(i-1)*3)]    # pick three dose levels
    # If the toxic effects are monotone, the sign of the three values y.tmp must be the same 
   # although small variation should be allowed. So adjustment can be helpful to avoid conflicting equations
   # In the present algorithm, adjustment is toward the average of the three   

    tmp<-sort(y.tmp)
    if (tmp[1]*tmp[3]<0){
	    if (tmp[2]*tmp[3]<0) y.tmp[y.tmp==max(y.tmp)]<--0.01
       else y.tmp[y.tmp==min(y.tmp)]<-0.01}   # theoretically the all y.mean must be of the same sign
                                          # so here is the adjustment
                                          # An alternative is not to make any adjustment 
   
    InitValMatrix[i,((baselinePoly+1):(baselinePoly+3))]<-
        try(ToxDiffSolve(y.tmp, Dose.tmp, Time.tmp-ExpoTime, K1=0, delta=10^-4, MaxIter=100))
       InitValMatrix[i,1:baselinePoly]<-A.hat
} # for i
  #  InitValMatrix[,1:baselinePoly]<-A.hat
  # name parameters for:      
    dimnames(InitValMatrix)[[2]]<-switch(baselinePoly, 
                                  c("A", "B", "C", "K"),
                                  c("A0","A1","B", "C", "K"), 
                                  c("A0", "A1", "A2", "B", "C", "K") 
                                  ) 
    InitValMatrix

} # end of ToxicoDiffInit

# defining the equation solving routine
# baseline has been substracted from Y

ToxDiffSolve<-function(Y, Dose, Time, K1=0, delta=10^-4, MaxIter=100){
	r<-0
	BC1<-rep(0,2)
	change<-1
	while((change > delta) && (r < MaxIter)){
		r<-r+1
		K0<-K1
		BC0<-BC1
		# solve for (B,C) with given K0
		x<-Dose*Time*exp(-K0*Time)
		x<-cbind(x, -Y*x)[1:2, 1:2]
		BC1<-solve(x, Y[1:2])
	  
	   # Solving for K0 from the third equation given (B,C) 
		
		if (Y[3]/((BC1[1]-BC1[2]*Y[3])*Dose[3]*Time[3])<=0) K1<-0.1
			      # This may be a useful adjustment so that initial value for B and C can be available 
					else {K1<--log(Y[3]/((BC1[1]-BC1[2]*Y[3])*Dose[3]*Time[3]))    
		                 K1<-K1/Time[3]}
		if (any(is.na(c(BC1, K1)))) break
	      else {change<-sqrt(sum( (c(BC1, K1)-c(BC0, K0))^2))/sqrt(sum((c(BC0, K0))^2))}
} # while 
	   if (change <= delta) c(BC1, K1) else rep(NA, 3)
} # ToxDiffSolve

# Creates matrix of possible initial values.
# Second to last row is medians of initial values from ToxicoDiffInit function
# Last row of matrix is means of initial values from ToxicoDiffInit function
# All other rows are initial values from ToxicoDiffInit function with NAs omited

ToxicoDiffInitMatrix<-function(dataset, ExpoTime=0, baselinePoly=1) {
	possible.init<-ToxicoDiffInit(dataset, ExpoTime, baselinePoly)
	possible.init<-possible.init[!is.na(possible.init[,4]),] #omit missing rows
	if(baselinePoly==1) {
		if(length(possible.init)>(3+baselinePoly)) {
		possible.init<-t(matrix(c(t(possible.init),
			median(possible.init[,1]),median(possible.init[,2]),median(possible.init[,3]),median(possible.init[,4]),
			mean(possible.init[,1]),mean(possible.init[,2]),mean(possible.init[,3]),mean(possible.init[,4])),nrow=4))}
	}
	if(baselinePoly==2) {
		if(length(possible.init)>(3+baselinePoly)) {
		possible.init<-t(matrix(c(t(possible.init),
			median(possible.init[,1]),median(possible.init[,2]),median(possible.init[,3]),median(possible.init[,4]),median(possible.init[,5]),
			mean(possible.init[,1]),mean(possible.init[,2]),mean(possible.init[,3]),mean(possible.init[,4]),mean(possible.init[,5])),nrow=5))}
	}
	if(baselinePoly==3) {
		if(length(possible.init)>(3+baselinePoly)) {
		possible.init<-t(matrix(c(t(possible.init),
			median(possible.init[,1]),median(possible.init[,2]),median(possible.init[,3]),median(possible.init[,4]),
			median(possible.init[,5]),median(possible.init[,6]),
			mean(possible.init[,1]),mean(possible.init[,2]),mean(possible.init[,3]),mean(possible.init[,4]),
			mean(possible.init[,5]), mean(possible.init[,6])),nrow=6))}
	}

	matrix(possible.init,ncol=3+baselinePoly)}


